"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectionPersistent = void 0;
const internal_1 = require("../internal");
class CollectionPersistent extends internal_1.Persistent {
    constructor(collection, config = {}) {
        super(collection.agileInstance(), {
            instantiate: false,
        });
        config = internal_1.defineConfig(config, {
            instantiate: true,
            storageKeys: [],
            defaultStorageKey: null,
        });
        this.collection = () => collection;
        this.instantiatePersistent({
            key: config.key,
            storageKeys: config.storageKeys,
            defaultStorageKey: config.defaultStorageKey,
        });
        if (this.ready && config.instantiate)
            this.initialLoading();
    }
    initialLoading() {
        const _super = Object.create(null, {
            initialLoading: { get: () => super.initialLoading }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.initialLoading.call(this).then(() => {
                this.collection().isPersisted = true;
            });
        });
    }
    loadPersistedValue(storageItemKey) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _storageItemKey = storageItemKey !== null && storageItemKey !== void 0 ? storageItemKey : this._key;
            const isPersisted = yield this.agileInstance().storages.get(_storageItemKey, this.config.defaultStorageKey);
            if (!isPersisted)
                return false;
            const loadValuesIntoCollection = () => __awaiter(this, void 0, void 0, function* () {
                var _a, _b, _c;
                const defaultGroup = this.collection().getDefaultGroup();
                if (defaultGroup == null)
                    return false;
                const defaultGroupStorageKey = CollectionPersistent.getGroupStorageKey(defaultGroup._key, _storageItemKey);
                defaultGroup.persist(defaultGroupStorageKey, {
                    loadValue: false,
                    defaultStorageKey: this.config.defaultStorageKey || undefined,
                    storageKeys: this.storageKeys,
                    followCollectionPersistKeyPattern: false,
                });
                if ((_a = defaultGroup.persistent) === null || _a === void 0 ? void 0 : _a.ready)
                    yield defaultGroup.persistent.initialLoading();
                for (const itemKey of defaultGroup._value) {
                    const item = this.collection().getItem(itemKey);
                    const itemStorageKey = CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
                    if (item != null) {
                        item.persist(itemStorageKey, {
                            loadValue: false,
                            defaultStorageKey: this.config.defaultStorageKey || undefined,
                            storageKeys: this.storageKeys,
                            followCollectionPersistKeyPattern: false,
                        });
                        if ((_b = item === null || item === void 0 ? void 0 : item.persistent) === null || _b === void 0 ? void 0 : _b.ready)
                            yield item.persistent.initialLoading();
                    }
                    else {
                        const placeholderItem = this.collection().getItemWithReference(itemKey);
                        placeholderItem === null || placeholderItem === void 0 ? void 0 : placeholderItem.persist(itemStorageKey, {
                            loadValue: false,
                            defaultStorageKey: this.config.defaultStorageKey,
                            storageKeys: this.storageKeys,
                            followCollectionPersistKeyPattern: false,
                        });
                        if ((_c = placeholderItem === null || placeholderItem === void 0 ? void 0 : placeholderItem.persistent) === null || _c === void 0 ? void 0 : _c.ready) {
                            const loadedPersistedValueIntoItem = yield placeholderItem.persistent.loadPersistedValue();
                            if (loadedPersistedValueIntoItem) {
                                this.collection().assignItem(placeholderItem, {
                                    overwrite: true,
                                });
                                placeholderItem.isPersisted = true;
                                this.collection().size += 1;
                            }
                        }
                    }
                }
                return true;
            });
            const success = yield loadValuesIntoCollection();
            if (success)
                this.setupSideEffects(_storageItemKey);
            return success;
        });
    }
    persistValue(storageItemKey) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _storageItemKey = storageItemKey !== null && storageItemKey !== void 0 ? storageItemKey : this._key;
            const defaultGroup = this.collection().getDefaultGroup();
            if (defaultGroup == null)
                return false;
            const defaultGroupStorageKey = CollectionPersistent.getGroupStorageKey(defaultGroup._key, _storageItemKey);
            this.agileInstance().storages.set(_storageItemKey, true, this.storageKeys);
            defaultGroup.persist(defaultGroupStorageKey, {
                defaultStorageKey: this.config.defaultStorageKey || undefined,
                storageKeys: this.storageKeys,
                followCollectionPersistKeyPattern: false,
            });
            for (const itemKey of defaultGroup._value) {
                const item = this.collection().getItem(itemKey);
                const itemStorageKey = CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
                item === null || item === void 0 ? void 0 : item.persist(itemStorageKey, {
                    defaultStorageKey: this.config.defaultStorageKey || undefined,
                    storageKeys: this.storageKeys,
                    followCollectionPersistKeyPattern: false,
                });
            }
            this.setupSideEffects(_storageItemKey);
            this.isPersisted = true;
            return true;
        });
    }
    setupSideEffects(storageItemKey) {
        const _storageItemKey = storageItemKey !== null && storageItemKey !== void 0 ? storageItemKey : this._key;
        const defaultGroup = this.collection().getDefaultGroup();
        if (defaultGroup == null)
            return;
        defaultGroup.addSideEffect(CollectionPersistent.defaultGroupSideEffectKey, (instance) => this.rebuildStorageSideEffect(instance, _storageItemKey), { weight: 0 });
    }
    removePersistedValue(storageItemKey) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _storageItemKey = storageItemKey !== null && storageItemKey !== void 0 ? storageItemKey : this._key;
            const defaultGroup = this.collection().getDefaultGroup();
            if (!defaultGroup)
                return false;
            const defaultGroupStorageKey = CollectionPersistent.getGroupStorageKey(defaultGroup._key, _storageItemKey);
            this.agileInstance().storages.remove(_storageItemKey, this.storageKeys);
            (_a = defaultGroup.persistent) === null || _a === void 0 ? void 0 : _a.removePersistedValue(defaultGroupStorageKey);
            defaultGroup.removeSideEffect(CollectionPersistent.defaultGroupSideEffectKey);
            for (const itemKey of defaultGroup._value) {
                const item = this.collection().getItem(itemKey);
                const itemStorageKey = CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
                (_b = item === null || item === void 0 ? void 0 : item.persistent) === null || _b === void 0 ? void 0 : _b.removePersistedValue(itemStorageKey);
            }
            this.isPersisted = false;
            return true;
        });
    }
    formatKey(key) {
        if (key == null && this.collection()._key)
            return this.collection()._key;
        if (key == null)
            return;
        if (this.collection()._key == null)
            this.collection()._key = key;
        return key;
    }
    rebuildStorageSideEffect(group, storageItemKey) {
        var _a;
        const collection = group.collection();
        const _storageItemKey = storageItemKey || ((_a = collection.persistent) === null || _a === void 0 ? void 0 : _a._key);
        if (group.previousStateValue.length === group._value.length)
            return;
        const addedKeys = group._value.filter((key) => !group.previousStateValue.includes(key));
        const removedKeys = group.previousStateValue.filter((key) => !group._value.includes(key));
        addedKeys.forEach((itemKey) => {
            const item = collection.getItem(itemKey);
            const itemStorageKey = CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
            if (item != null && !item.isPersisted)
                item.persist(itemStorageKey, {
                    defaultStorageKey: this.config.defaultStorageKey || undefined,
                    storageKeys: this.storageKeys,
                    followCollectionPersistKeyPattern: false,
                });
        });
        removedKeys.forEach((itemKey) => {
            var _a;
            const item = collection.getItem(itemKey);
            const itemStorageKey = CollectionPersistent.getItemStorageKey(itemKey, _storageItemKey);
            if (item != null && item.isPersisted)
                (_a = item.persistent) === null || _a === void 0 ? void 0 : _a.removePersistedValue(itemStorageKey);
        });
    }
    static getItemStorageKey(itemKey, collectionKey) {
        if (itemKey == null || collectionKey == null)
            internal_1.LogCodeManager.log('1A:02:00');
        if (itemKey == null)
            itemKey = 'unknown';
        if (collectionKey == null)
            collectionKey = 'unknown';
        return this.storageItemKeyPattern
            .replace('${collectionKey}', collectionKey.toString())
            .replace('${itemKey}', itemKey.toString());
    }
    static getGroupStorageKey(groupKey, collectionKey) {
        if (groupKey == null || collectionKey == null)
            internal_1.LogCodeManager.log('1A:02:01');
        if (groupKey == null)
            groupKey = 'unknown';
        if (collectionKey == null)
            collectionKey = 'unknown';
        return this.storageGroupKeyPattern
            .replace('${collectionKey}', collectionKey.toString())
            .replace('${groupKey}', groupKey.toString());
    }
}
exports.CollectionPersistent = CollectionPersistent;
CollectionPersistent.defaultGroupSideEffectKey = 'rebuildGroupStorageValue';
CollectionPersistent.storageItemKeyPattern = '_${collectionKey}_item_${itemKey}';
CollectionPersistent.storageGroupKeyPattern = '_${collectionKey}_group_${groupKey}';
