"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Persistent = void 0;
const internal_1 = require("../internal");
class Persistent {
    constructor(agileInstance, config = {}) {
        this.ready = false;
        this.isPersisted = false;
        this.storageKeys = [];
        this.agileInstance = () => agileInstance;
        this._key = Persistent.placeHolderKey;
        config = internal_1.defineConfig(config, {
            instantiate: true,
            storageKeys: [],
            defaultStorageKey: null,
        });
        this.agileInstance().storages.persistentInstances.add(this);
        this.config = { defaultStorageKey: config.defaultStorageKey };
        if (config.instantiate) {
            this.instantiatePersistent({
                storageKeys: config.storageKeys,
                key: config.key,
                defaultStorageKey: config.defaultStorageKey,
            });
        }
    }
    set key(value) {
        this.setKey(value);
    }
    get key() {
        return this._key;
    }
    setKey(value) {
        return __awaiter(this, void 0, void 0, function* () {
            const oldKey = this._key;
            const wasReady = this.ready;
            if (value === this._key)
                return;
            this._key = value !== null && value !== void 0 ? value : Persistent.placeHolderKey;
            const isValid = this.validatePersistent();
            if (!wasReady) {
                if (isValid)
                    yield this.initialLoading();
                return;
            }
            yield this.removePersistedValue(oldKey);
            if (isValid)
                yield this.persistValue(value);
        });
    }
    instantiatePersistent(config = {}) {
        var _a;
        this._key = (_a = this.formatKey(config.key)) !== null && _a !== void 0 ? _a : Persistent.placeHolderKey;
        this.assignStorageKeys(config.storageKeys, config.defaultStorageKey);
        this.validatePersistent();
    }
    validatePersistent() {
        let isValid = true;
        if (this._key === Persistent.placeHolderKey) {
            internal_1.LogCodeManager.log('12:03:00');
            isValid = false;
        }
        if (this.config.defaultStorageKey == null || this.storageKeys.length <= 0) {
            internal_1.LogCodeManager.log('12:03:01');
            isValid = false;
        }
        this.storageKeys.map((key) => {
            if (!this.agileInstance().storages.storages[key]) {
                internal_1.LogCodeManager.log('12:03:02', [this._key, key]);
                isValid = false;
            }
        });
        this.ready = isValid;
        return isValid;
    }
    assignStorageKeys(storageKeys = [], defaultStorageKey) {
        const storages = this.agileInstance().storages;
        const _storageKeys = internal_1.copy(storageKeys);
        if (defaultStorageKey != null && !_storageKeys.includes(defaultStorageKey))
            _storageKeys.push(defaultStorageKey);
        if (_storageKeys.length <= 0) {
            const defaultStorageKey = storages.config.defaultStorageKey;
            if (defaultStorageKey != null) {
                this.config.defaultStorageKey = defaultStorageKey;
                _storageKeys.push(storages.config.defaultStorageKey);
            }
        }
        else {
            this.config.defaultStorageKey = defaultStorageKey !== null && defaultStorageKey !== void 0 ? defaultStorageKey : _storageKeys[0];
        }
        this.storageKeys = _storageKeys;
    }
    initialLoading() {
        return __awaiter(this, void 0, void 0, function* () {
            const success = yield this.loadPersistedValue();
            if (this.onLoad)
                this.onLoad(success);
            if (!success)
                yield this.persistValue();
        });
    }
    loadPersistedValue(storageItemKey) {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.LogCodeManager.log('00:03:00', ['loadPersistedValue', 'Persistent']);
            return false;
        });
    }
    persistValue(storageItemKey) {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.LogCodeManager.log('00:03:00', ['persistValue', 'Persistent']);
            return false;
        });
    }
    removePersistedValue(storageItemKey) {
        return __awaiter(this, void 0, void 0, function* () {
            internal_1.LogCodeManager.log('00:03:00', ['removePersistedValue', 'Persistent']);
            return false;
        });
    }
    formatKey(key) {
        return key;
    }
}
exports.Persistent = Persistent;
Persistent.placeHolderKey = '__THIS_IS_A_PLACEHOLDER__';
