"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runsOnServer = exports.globalBind = exports.optionalRequire = exports.extractRelevantObservers = exports.extractObservers = exports.getAgileInstance = void 0;
const internal_1 = require("./internal");
function getAgileInstance(instance) {
    try {
        if (instance) {
            const _agileInstance = internal_1.isFunction(instance['agileInstance'])
                ? instance['agileInstance']()
                : instance['agileInstance'];
            if (_agileInstance)
                return _agileInstance;
        }
        if (internal_1.shared instanceof internal_1.Agile) {
            return internal_1.shared;
        }
        return globalThis[internal_1.Agile.globalKey];
    }
    catch (e) {
        internal_1.LogCodeManager.log('20:03:00', [], instance);
    }
    return undefined;
}
exports.getAgileInstance = getAgileInstance;
function extractObservers(instances) {
    const observers = [];
    const tempInstancesArray = internal_1.normalizeArray(instances, {
        createUndefinedArray: true,
    });
    for (const instance of tempInstancesArray) {
        if (instance == null) {
            observers.push({});
            continue;
        }
        if (instance instanceof internal_1.Collection) {
            observers.push(instance.getGroupWithReference(instance.config.defaultGroupKey)
                .observers);
            continue;
        }
        if (instance['observer'] && instance['observer'] instanceof internal_1.Observer) {
            observers.push({ value: instance['observer'] });
            continue;
        }
        if (instance['observers']) {
            const extractedObservers = {};
            for (const key in instance['observers']) {
                if (instance['observers'][key] instanceof internal_1.Observer) {
                    extractedObservers[key] = instance['observers'][key];
                }
            }
            observers.push(extractedObservers);
            continue;
        }
        if (instance instanceof internal_1.Observer) {
            observers.push({ value: instance });
            continue;
        }
        observers.push({});
    }
    return Array.isArray(instances) ? observers : observers[0];
}
exports.extractObservers = extractObservers;
function extractRelevantObservers(instances, observerType) {
    var _a;
    const depsWithIndicator = {};
    const depsWithNoIndicator = [];
    for (const depKey in instances) {
        const extractedObservers = extractObservers(instances[depKey]);
        let observer = undefined;
        if (observerType != null && extractedObservers[observerType] != null)
            observer = extractedObservers[observerType];
        if (observerType == null)
            observer = (_a = extractedObservers['output']) !== null && _a !== void 0 ? _a : extractedObservers['value'];
        if (Array.isArray(instances))
            depsWithNoIndicator.push(observer);
        else
            depsWithIndicator[depKey] = observer;
    }
    return Array.isArray(instances) ? depsWithNoIndicator : depsWithIndicator;
}
exports.extractRelevantObservers = extractRelevantObservers;
function optionalRequire(moduleName, error = true) {
    let requiredPackage = null;
    try {
        requiredPackage = require(moduleName);
    }
    catch (e) {
        if (error) {
            internal_1.LogCodeManager.log('20:03:02', [moduleName]);
            console.log(e);
        }
    }
    return requiredPackage;
}
exports.optionalRequire = optionalRequire;
function globalBind(key, instance, overwrite = false) {
    try {
        if (overwrite) {
            globalThis[key] = instance;
            return true;
        }
        if (globalThis[key] == null) {
            globalThis[key] = instance;
            return true;
        }
    }
    catch (e) {
        internal_1.LogCodeManager.log('20:03:01', [key]);
    }
    return false;
}
exports.globalBind = globalBind;
const runsOnServer = () => {
    return !(typeof window !== 'undefined' &&
        typeof window.document !== 'undefined' &&
        typeof window.document.createElement !== 'undefined');
};
exports.runsOnServer = runsOnServer;
